package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDetailDto;
import com.qmth.distributed.print.business.bean.dto.FieldsDto;
import com.qmth.distributed.print.business.bean.result.ExaminationDetailResult;
import com.qmth.distributed.print.business.bean.result.ExaminationResult;
import com.qmth.distributed.print.business.bean.result.SummarizedDataResult;
import com.qmth.distributed.print.business.entity.BasicExamRule;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.entity.ExamPrintPlan;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamDetailMapper;
import com.qmth.distributed.print.business.service.BasicExamRuleService;
import com.qmth.distributed.print.business.service.CacheService;
import com.qmth.distributed.print.business.service.ExamDetailCourseService;
import com.qmth.distributed.print.business.service.ExamDetailService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 考务数据 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamDetailServiceImpl extends ServiceImpl<ExamDetailMapper, ExamDetail> implements ExamDetailService {
    @Resource
    private ExamDetailCourseService examDetailCourseService;
    @Resource
    private BasicExamRuleService basicExamRuleService;
    @Resource
    private ExamDetailMapper examDetailMapper;
    @Resource
    private CacheService cacheService;

    private final static Logger log = LoggerFactory.getLogger(ExamDetailServiceImpl.class);

    @Transactional(rollbackFor = Exception.class)
    @Override
    public double calculateTotalSubjects(Long printPlanId) {
        List<ExamDetail> examDetailList = this.list(new QueryWrapper<ExamDetail>().lambda().eq(ExamDetail::getPrintPlanId,printPlanId));
        return examDetailList.stream().collect(Collectors.summarizingDouble(ExamDetail::getTotalSubjects)).getSum();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public double calculateTotalPackages(Long printPlanId) {
        double result = 0;
        List<ExamDetail> examDetailList = this.list(new QueryWrapper<ExamDetail>().lambda().eq(ExamDetail::getPrintPlanId,printPlanId));
        for (ExamDetail examDetail : examDetailList) {
            double count = examDetailCourseService.calculatePackagesByDetailId(examDetail.getId());
            result = result + count;
        }
        return result;
    }

    @Override
    public IPage<ExamTaskDetailDto> listPrintTask(String printPlanName, String status, String courseCode, String paperNumber, String examPlace, Long examStartTime, Long examEndTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<ExamTaskDetailDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDetailDto> examTaskDetailDtoIPage = this.baseMapper.listPrintTask(page, schoolId, printPlanName,status, courseCode, paperNumber, examPlace, examStartTime, examEndTime);
        return examTaskDetailDtoIPage;
    }

    @Override
    public Map<String, Object> taskTotalData(String printPlanName, String status, String courseCode, String paperNumber, String examPlace, Long examStartTime, Long examEndTime) {
        return null;
    }

    @Override
    public String taskViewPDF(Long id) {
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<FieldsDto> findExaminationFields(Long schoolId) {
        // TODO: 2021/4/9 回复schoolId的获取方式
//        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId();
        BasicExamRule basicExamRule = basicExamRuleService.list(new QueryWrapper<BasicExamRule>().lambda().eq(BasicExamRule::getSchoolId,schoolId)).get(0);
        if (basicExamRule == null){
            throw ExceptionResultEnum.ERROR.exception("找不到该学校考务字段信息 + schoolId" + schoolId);
        }
        String requiredFields = basicExamRule.getRequiredFields();
        String extendFields = basicExamRule.getExtendFields();
        // 必选字段
        List<FieldsDto> requiredFieldsList = JSONObject.parseArray(requiredFields,FieldsDto.class);
        if (requiredFieldsList.stream().anyMatch(e -> !e.getEnable())){
            throw ExceptionResultEnum.ERROR.exception("该学校考务字段设置存在必选字段禁用的情况 schoolId = " + schoolId);
        }
        for (FieldsDto fieldsDto : requiredFieldsList) {
            fieldsDto.setLevel("primary");
        }

        // 扩展字段
        List<FieldsDto> extendFieldsList = JSONObject.parseArray(extendFields,FieldsDto.class);
        List<String> extendCodeList = extendFieldsList.stream().map(FieldsDto::getCode).collect(Collectors.toList());
        if (extendCodeList.stream().anyMatch(e -> requiredFieldsList.stream().map(FieldsDto::getCode).collect(Collectors.toList()).contains(e))){
            throw ExceptionResultEnum.ERROR.exception("该学校考务字段设置存在相同的code schoolId = " + schoolId);
        }
        // 有效的扩展字段
        List<FieldsDto> validExtendList = extendFieldsList.stream().filter(FieldsDto::getEnable).collect(Collectors.toList());
        for (FieldsDto fieldsDto : validExtendList) {
            fieldsDto.setLevel("secondary");
        }

        // 有效的考务字段
        List<FieldsDto> validExaminationFieldList = new ArrayList<>();
        validExaminationFieldList.addAll(requiredFieldsList);
        validExaminationFieldList.addAll(validExtendList);
        System.out.println("---" + validExaminationFieldList);

        List<String> fieldsName = validExaminationFieldList.stream().map(FieldsDto::getName).collect(Collectors.toList());
        for (String s : fieldsName) {
            System.out.println(s);
        }
        return validExaminationFieldList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void downLoadExaminationTemplate(Long schoolId, HttpServletResponse response) throws IOException {
        final String DEFALUT_EXT = ".xlsx";
        final String DEFALUT_CONTENT_TYPE = "application/vnd.ms-excel";
        List<String> fieldsNameList = this.findExaminationFields(schoolId).stream().map(FieldsDto::getName).distinct().collect(Collectors.toList());

        log.debug("导出Excel开始...");
        HSSFWorkbook wb = new HSSFWorkbook();
        String fileName = "考务数据模板";
        response.setHeader("Content-Disposition", "inline;filename=" + URLEncoder.encode(fileName, SystemConstant.CHARSET_NAME) + DEFALUT_EXT);
        response.setContentType(DEFALUT_CONTENT_TYPE);
        HSSFSheet sheet = wb.createSheet(cacheService.schoolCache(schoolId).getName() + "考务数据模板");
        sheet.setDefaultRowHeight((short) (2 * 256));
        sheet.setColumnWidth(0, 50 * 160);

        HSSFFont font = wb.createFont();
        font.setFontHeightInPoints((short) 16);

        HSSFRow row = sheet.createRow((int) 0);
        HSSFCellStyle style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER);

        for(int i = 0; i<fieldsNameList.size(); i++){
            HSSFCell cell = row.createCell(i);
            cell.setCellValue(fieldsNameList.get(i));
            cell.setCellStyle(style);
        }
        try {
            ServletOutputStream outputStream = response.getOutputStream();
            wb.write(outputStream);
            outputStream.close();
            log.debug("导出Excel结束...");
        }
        catch (IOException e){

        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<ExaminationResult> findExaminationBriefPage(Long schoolId, Long printPlanId, String courseCode, String paperNumber, String examPlace, String examRoom, String packageCode, int pageNumber, int pageSize) {
        IPage<ExaminationResult> page = examDetailMapper.findBriefPage(new Page<>(pageNumber,pageSize),schoolId,printPlanId,courseCode,paperNumber,examPlace,examRoom,packageCode);
        List<ExaminationResult> list = page.getRecords();
        for (ExaminationResult examinationResult : list) {
            String examDetailCourseIds = examinationResult.getExamDetailCourseIds();
            if (examDetailCourseIds == null || examDetailCourseIds.length() == 0){
                throw ExceptionResultEnum.ERROR.exception("考务数据确实课程信息");
            }
            String[] arr = examDetailCourseIds.split(",");

            final String separator = "、";
            String courseNameCodeStr = "";
            String paperNumberStr = "";
            for (String examDetailCourseId : arr) {
                ExamDetailCourse examDetailCourse = examDetailCourseService.getById(examDetailCourseId);
                String code = examDetailCourse.getCourseCode();
                String name = examDetailCourse.getCourseName();
                String number = examDetailCourse.getPaperNumber();
                courseNameCodeStr = courseNameCodeStr + name + "(" + code + ")" + separator;
                paperNumberStr = paperNumberStr + number + separator;
            }
            courseNameCodeStr = courseNameCodeStr.substring(0,courseNameCodeStr.length() - separator.length());
            paperNumberStr = paperNumberStr.substring(0,paperNumberStr.length() - separator.length());
            examinationResult.setCourseNameCode(courseNameCodeStr);
            examinationResult.setPaperNumber(paperNumberStr);
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public SummarizedDataResult findSummarizedData(Long schoolId, Long printPlanId, String courseCode, String paperNumber, String examPlace, String examRoom, String packageCode) {
        IPage<ExaminationResult> page = this.findExaminationBriefPage(schoolId, printPlanId, courseCode, paperNumber, examPlace, examRoom, packageCode, SystemConstant.PAGE_NUMBER, SystemConstant.PAGE_SIZE);
        List<ExaminationResult> list = page.getRecords();
        int totalSubjects = 0;
        int packageCount = 0;
        for (ExaminationResult examinationResult : list) {
            int subjects = examinationResult.getTotalSubjects();
            int packages = examinationResult.getCourseNameCode().split("、").length;
            totalSubjects = totalSubjects + subjects;
            packageCount = packageCount + packages;
        }
        SummarizedDataResult summarizedDataResult = new SummarizedDataResult();
        summarizedDataResult.setSchoolId(schoolId);
        summarizedDataResult.setTotalSubjects(totalSubjects);
        summarizedDataResult.setPackageCount(packageCount);
        return summarizedDataResult;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<ExaminationDetailResult> findExaminationDetail(Long printPlanId, String courseCode, String paperNumber, String examPlace, String examRoom, String studentParam, int pageNumber, int pageSize) {
        return examDetailMapper.findDetailPage(new Page<>(pageNumber,pageSize),printPlanId,courseCode,paperNumber,examPlace,examRoom,studentParam);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> findExamPlaceDatasource() {
        List<ExamDetail> examDetail = this.list();
        return examDetail.stream().map(ExamDetail::getExamPlace).distinct().collect(Collectors.toList());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> findExamRoomDatasource() {
        List<ExamDetail> examDetail = this.list();
        return examDetail.stream().map(ExamDetail::getExamRoom).distinct().collect(Collectors.toList());
    }
}
